"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.getExternalModulePath = exports.parseExternalModuleName = exports.toModulePath = void 0;
const fs = require("fs");
const path_1 = require("path");
const util_1 = require("util");
const fs_utils_1 = require("./fs-utils");
const readFile = util_1.promisify(fs.readFile);
function toModulePath(options) {
    if (parseExternalModuleName(options.from))
        return options.from;
    let from = options.removeExtensions
        ? removeExtensions(options.from, options.removeExtensions)
        : options.from;
    if (!from.startsWith("./") && !from.startsWith("../"))
        throw new Error(`Module path must start with './' or '../'`);
    let parentDir = options.relativeToModule ? path_1.dirname(options.relativeToModule) : ".";
    while (true) {
        if (from.startsWith("./"))
            from = from.substr(2);
        else if (from.startsWith("../")) {
            const newDir = path_1.dirname(parentDir);
            if (newDir === parentDir)
                break;
            parentDir = newDir;
            from = from.substr(3);
        }
        else
            break;
    }
    if (from.startsWith("../"))
        return path_1.normalize(from);
    const result = `${parentDir}/${from}`;
    if (result.startsWith("./"))
        return "./" + path_1.normalize(result);
    return path_1.normalize(result);
}
exports.toModulePath = toModulePath;
function parseExternalModuleName(modulePath) {
    const result = /^((?:@[a-z0-9-~][a-z0-9-\._~]*\/)?[a-z0-9-~][a-z0-9-\._~]*)(\/.+)?$/.exec(modulePath);
    if (!result)
        return;
    return {
        packageName: result[1],
        additionalPath: result[2],
    };
}
exports.parseExternalModuleName = parseExternalModuleName;
async function getExternalModulePath(parsed, sourceDir) {
    var _a;
    // console.log("==>", parsed, sourceDir)
    const mainPackageDir = await findParentPackageDir(sourceDir);
    // console.log("==. packageDir", mainPackageDir)
    if (!mainPackageDir)
        throw new Error(`Cannot find the package directory of '${sourceDir}'`);
    const depPackageDir = path_1.join(mainPackageDir, "node_modules", parsed.packageName);
    if (parsed.additionalPath)
        return path_1.join(depPackageDir, parsed.additionalPath);
    const content = JSON.parse(await readFile(path_1.join(depPackageDir, "package.json"), "utf8"));
    const additionalPath = (_a = content.types) !== null && _a !== void 0 ? _a : content.typing;
    if (additionalPath)
        return path_1.join(depPackageDir, removeModuleNameExtension(additionalPath));
    return path_1.join(depPackageDir, "index");
}
exports.getExternalModulePath = getExternalModulePath;
function removeModuleNameExtension(fileName) {
    if (fileName.endsWith(".ts"))
        fileName = fileName.substring(0, fileName.length - (fileName.endsWith(".d.ts") ? 5 : 3));
    return fileName;
}
async function findParentPackageDir(dir) {
    dir = path_1.resolve(dir);
    while (true) {
        if (await fs_utils_1.pathExists(path_1.join(dir, "package.json")))
            return dir;
        if (dir === "/" || dir === ".")
            return;
        dir = path_1.dirname(dir);
    }
}
function removeExtensions(path, extensions) {
    for (const extension of extensions) {
        if (path.endsWith(extension))
            return path.substr(0, path.length - extension.length);
    }
    return path;
}
